<?php

namespace App\Http\Controllers;

use App\Events\MailSendEvent;
use App\Http\Requests\UserStoreRequest;
use App\Http\Requests\UserUpdateRequest;
use App\Http\Resources\UserResource;
use App\Models\User;
use App\Repositories\AccountActivationRepository;
use App\Repositories\CourseRepository;
use App\Repositories\FcmDeviceTokenRepository;
use App\Repositories\UserRepository;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;

class UserController extends Controller
{
    public function register(UserStoreRequest $request)
    {
        $newUser = UserRepository::storeByRequest($request);
        $token = $newUser->createToken('auth_token')->accessToken;

        if ($request->fcm_token) {
            FcmDeviceTokenRepository::create([
                'user_id' => $newUser->id,
                'token' => $request->fcm_token,
            ]);
        }

        $code = rand(1111, 9999);

        AccountActivationRepository::create([
            'user_id' => $newUser->id,
            'code' => $code,
            'valid_until' => now()->addHour(),
        ]);

        try {
            MailSendEvent::dispatch($code, $newUser->email);
        } catch (\Exception $e) {
        }

        if ($request->guest_id) {
            CourseRepository::syncFavourites($request->guest_id, $newUser);
        }

        $newUser->refresh();

        return $this->json('Registration successful', [
            'user' => UserResource::make($newUser),
            'token' => $token,
            'activation_code' => $code,
        ], 201);
    }

    public function login(Request $request)
    {
        $validatedData = $request->validate([
            'email' => 'required_without:phone',
            'phone' => 'required_without:email',
            'password' => 'required',
        ]);

        if (Auth::attempt($validatedData)) {
            /** @var User */
            $user = Auth::user();

            $token = $user->createToken('auth_token')->accessToken;

            if ($request->fcm_token) {
                FcmDeviceTokenRepository::create([
                    'user_id' => $user->id,
                    'token' => $request->fcm_token,
                ]);
            }

            if ($request->guest_id) {
                CourseRepository::syncFavourites($request->guest_id, $user);
            }

            return $this->json('Login successful', [
                'user'  => UserResource::make($user),
                'token' => $token,
            ]);
        }

        return $this->json('Invalid credentials', null, 403);
    }

    public function index()
    {
        return $this->json('User data found', [
            'user' => UserResource::make(auth()->user()),
        ]);
    }

    public function update(UserUpdateRequest $request)
    {
        /** @var User */
        $user = auth()->user();

        if ($request->current_password) {
            if (!Hash::check($request->current_password, $user->password)) {
                return $this->json('Current password is incorrect', null, 422);
            }
        }

        UserRepository::updateByRequest($request, auth()->user());
        $updatedUser = UserRepository::find(auth()->user()->id);

        return $this->json('User data updated', [
            'user' => UserResource::make($updatedUser),
        ]);
    }

    public function updatePassword(Request $request)
    {
        /** @var User */
        $user = auth()->user();
        $request->validate(['password' => 'min:8|confirmed']);

        UserRepository::update($user, ['password' => Hash::make($request->password)]);

        return $this->json('Password updated', null, 200);
    }

    public function delete()
    {
        /** @var User */
        $user = auth()->user();
        $user->delete();
        return $this->json('User deleted', null, 200);
    }
}
