<?php

namespace App\Repositories;

use Abedin\Boiler\Repositories\Repository;
use App\Enum\MediaTypeEnum;
use App\Http\Requests\ChapterStoreRequest;
use App\Http\Requests\ChapterUpdateRequest;
use App\Models\Chapter;
use Owenoj\LaravelGetId3\GetId3;

class ChapterRepository extends Repository
{
    public static function model()
    {
        return Chapter::class;
    }

    public static function storeByRequest(ChapterStoreRequest $request)
    {
        $chapter = self::create([
            'title' => $request->title,
            'serial_number' => $request->serial_number,
            'course_id' => $request->course_id,
        ]);

        foreach ($request->contents as $requestContent) {
            $contentMedia = isset($requestContent['media']) ? MediaRepository::storeByRequest(
                $requestContent['media'],
                'course/chapter/content/media',
                MediaTypeEnum::IMAGE
            ) : null;

            if (isset($requestContent['is_forwardable'])) {
                $isForwardable = $requestContent['is_forwardable'] == 'on' ? true : false;
            } else {
                $isForwardable = false;
            }

            ContentRepository::create([
                'chapter_id' => $chapter->id,
                'media_id' => $contentMedia ? $contentMedia->id : null,
                'title' => $requestContent['title'],
                'type' => self::getFileType($requestContent['media']),
                'duration' => self::getMediaPlaytime($requestContent['media']),
                'serial_number' => $requestContent['serial_number'],
                'is_forwardable' => $isForwardable,
                'media_updated_at' => now()
            ]);
        }

        return $chapter;
    }

    public static function updateByRequest(ChapterUpdateRequest $request, Chapter $chapter)
    {
        self::update($chapter, [
            'title' => $request->title ?? $chapter->title,
            'serial_number' => $request->serial_number ?? $chapter->serial_number
        ]);

        // Delete removed content
        $existingContentIds = ContentRepository::query()->where('chapter_id', $chapter->id)->pluck('id')->toArray();
        $deletedContentIds = array_diff($existingContentIds, collect($request->contents)->pluck('content_id')->toArray());

        if ($deletedContentIds) {
            ContentRepository::query()->whereIn('id', $deletedContentIds)->delete();
        }

        $newContent = false;

        foreach ($request->contents as $content) {
            $contentId = isset($content['content_id']) ? $content['content_id'] : 0;

            if (!$contentId) {
                $media = self::uploadFile($content['media']);
                $mediaType = self::getFileType($content['media']);
                $mediaID = $media->id;
                $newContent = true;
            } else {
                $existsContent = ContentRepository::find($content['content_id']);
                $mediaID = $existsContent->media_id;
                $newMedia = false;
                $duration = 0;

                $mediaType = $existsContent->type;

                if (isset($content['media'])) {
                    $mediaType = self::getFileType($content['media']);
                    $media = self::uploadFile($content['media']);
                    $mediaID = $media->id;
                    $newMedia = true;
                }
            }

            $duration =  self::getMediaPlaytime($content['media']);

            if (isset($content['is_forwardable'])) {
                $isForwardable = $content['is_forwardable'] == 'on' ? true : false;
            } else {
                $isForwardable = false;
            }

            ContentRepository::query()->updateOrCreate([
                'id' => $contentId,
                'chapter_id' => $chapter->id
            ], [
                'title' => $content['title'],
                'type' => $mediaType,
                'duration' => $duration,
                'serial_number' => $content['serial_number'],
                'media_id' => $mediaID,
                'is_forwardable' => $isForwardable,
                'media_updated_at' => $newMedia ? now() : $existsContent->media_updated_at
            ]);
        }

        return $newContent;
    }

    private static function uploadFile($file)
    {
        return $file ? MediaRepository::storeByRequest(
            $file,
            'course/chapter/content/media',
            self::getFileType($file),
        ) : null;
    }

    private static function getMediaPlaytime($file)
    {
        $mediaType = self::getFileType($file);

        $minutes = 0;

        if ($mediaType == MediaTypeEnum::AUDIO || $mediaType == MediaTypeEnum::VIDEO) {
            $track = GetId3::fromUploadedFile($file);
            $time = explode(':', $track->getPlaytime());
            $minutes = (int) $time[0] ? $time[0] : 1;
        }

        return $minutes;
    }

    private static function getFileType($file)
    {
        switch ($file->getClientMimeType()) {
            case 'image/jpeg':
            case 'image/png':
            case 'image/jpg':
            case 'image/gif':
            case 'image/svg+xml':
                $mediaType = MediaTypeEnum::IMAGE;
                break;
            case 'video/mp4':
            case 'video/mpeg':
                $mediaType = MediaTypeEnum::VIDEO;
                break;
            case 'audio/mpeg':
            case 'audio/wav':
            case 'audio/webm':
            case 'audio/ogg':
            case 'audio/x-wav':
                $mediaType = MediaTypeEnum::AUDIO;
                break;
            case 'application/pdf':
            case 'application/msword':
            case 'application/vnd.openxmlformats-officedocument.wordprocessingml.document':
                $mediaType = MediaTypeEnum::DOCUMENT;
                break;
            default:
                $mediaType = MediaTypeEnum::IMAGE;
                break;
        }

        return $mediaType;
    }
}
