<?php

namespace App\Repositories;

use Abedin\Boiler\Repositories\Repository;
use App\Enum\MediaTypeEnum;
use App\Http\Requests\CourseStoreRequest;
use App\Http\Requests\CourseUpdateRequest;
use App\Models\Course;
use App\Models\User;

class CourseRepository extends Repository
{
    public static function model()
    {
        return Course::class;
    }

    public static function storeByRequest(CourseStoreRequest $request)
    {
        if (isset($request->is_active)) {
            $isActive = $request->is_active == 'on' ? true : false;
        } else {
            $isActive = false;
        }

        $media = $request->hasFile('media') ? MediaRepository::storeByRequest(
            $request->file('media'),
            'course/thumbnail',
            MediaTypeEnum::IMAGE
        ) : null;

        $video = $request->hasFile('video') ? MediaRepository::storeByRequest(
            $request->file('video'),
            'course/video',
            MediaTypeEnum::VIDEO
        ) : null;

        $course = self::create([
            'category_id' => $request->category_id,
            'title' => $request->title,
            'media_id' => $media ? $media->id : null,
            'video_id' => $video ? $video->id : null,
            'description' => json_encode($request->description),
            'regular_price' => $request->regular_price,
            'price' => $request->price,
            'instructor_id' => $request->instructor_id,
            'is_active' => $isActive,
            'published_at' => $request->is_active ? now() : null
        ]);

        foreach ($request->chapters ?? [] as $requestChapter) {
            $chapter = ChapterRepository::create([
                'title' => $requestChapter['title'],
                'serial_number' => $requestChapter['serial_number'],
                'course_id' => $course->id
            ]);

            foreach ($requestChapter['contents'] as $requestContent) {
                $contentMedia = isset($requestContent['media']) ? MediaRepository::storeByRequest(
                    $requestContent['media'],
                    'course/chapter/content/media',
                    MediaTypeEnum::IMAGE
                ) : null;

                ContentRepository::create([
                    'chapter_id' => $chapter->id,
                    'media_id' => $contentMedia ? $contentMedia->id : null,
                    'title' => $requestContent['title'],
                    'type' => $requestContent['type'],
                    'serial_number' => $requestContent['serial_number']
                ]);
            }
        }

        return $course;
    }

    public static function updateByRequest(CourseUpdateRequest $request, Course $course)
    {
        if (isset($request->is_active)) {
            $isActive = $request->is_active == 'on' ? true : false;
        } else {
            $isActive = false;
        }

        if ($course->media) {
            $media = $request->hasFile('media') ? MediaRepository::updateByRequest(
                $request->file('media'),
                $course->image,
                'course/thumbnail',
                MediaTypeEnum::IMAGE
            ) : $course->image;
        } else {
            $media = $request->hasFile('media') ? MediaRepository::storeByRequest(
                $request->file('media'),
                'course/thumbnail',
                MediaTypeEnum::IMAGE
            ) : null;
        }

        if ($course->video) {
            $video = $request->hasFile('video') ? MediaRepository::updateByRequest(
                $request->file('video'),
                $course->image,
                'course/video',
                MediaTypeEnum::VIDEO
            ) : $course->video;
        } else {
            $video = $request->hasFile('video') ? MediaRepository::storeByRequest(
                $request->file('video'),
                'course/video',
                MediaTypeEnum::VIDEO
            ) : null;
        }

        return self::update($course, [
            'category_id' => $request->category_id ?? $course->category_id,
            'title' => $request->title ?? $course->title,
            'media_id' => $media ? $media->id : $course->media->id,
            'video_id' => $video ? $video->id : null,
            'description' => json_encode($request->description) ?? $course->description,
            'regular_price' => $request->regular_price ?? null,
            'price' => $request->price ?? $course->price,
            'instructor_id' => $request->instructor_id ?? $course->instructor_id,
            'is_active' => $isActive,
            'published_at' => null == $course->published_at ? ($request->is_active ? now() : null) : $course->published_at
        ]);
    }

    public static function syncFavourites(string $guestUniqueId, User $user)
    {
        $guest = GuestRepository::query()->where('unique_id', '=', $guestUniqueId)->first();

        if ($guest) {
            $user->favouriteCourses()->attach($guest->favouriteCourses->pluck('id'));
            $guest->delete();
        }
    }
}
